@@ -1,36 +1,134 @@
 from . import attribute, exception, requests, utils
-from .type import Any, ClientType, Dict, Enum, EnumMeta, List, Response, Union
+from .type import (Any, ClientType, Dict, Enum, EnumMeta, List, Optional, Response, Union)
 
 
 class Client(ClientType):
-    '''    
-    This class acts as a MyKu API wrapper.
-    After created, the client will be logged in automatically.
+    '''Represents a client connection that connects to MyKU. This class is used to interact with the MyKU API.
+    The client is initialized with a username and password and will login to MyKU automatically.
     '''
 
     def __init__(self, username: str, password: str) -> None:
         self.__username: str = username
         self.__password: str = password
+        self.__login_response: Optional[dict] = {}
+        self.__access_token: Optional[str] = ''
+        self.__schedule_response: Optional[dict] = {}
+        self.__academic_year: Optional[str] = None
+        self.__semester: Optional[str] = None
 
         self.initialize()
 
-    def __valid_response(self,
-                         response: Response,
-                         to_json: bool = True) -> Union[dict, Response]:
-        '''If the response is not 200, check if the error is due to an expired token. 
-        If so, reset the token and raise an error. Otherwise, raise the error
-        
-        Parameters
-        ----------
-        response : (Response)
-            The response object returned from the request.
-        to_json : (bool, optional)
-            If True, the response will be converted to a JSON object.
-        
+    @property
+    def login_response(self) -> dict:
+        '''Recent login response.
+
         Returns
         -------
-            dict | Response 
-        
+        dict
+            Response from the login request.
+        '''
+        return self.__login_response
+
+    @property
+    def access_token(self) -> str:
+        '''Recent access token from the login response.
+
+        Returns
+        -------
+        str
+            Access token.
+        '''
+        return self.__access_token
+
+    @property
+    def schedule_response(self) -> dict:
+        '''Recent schedule response.
+
+        Returns
+        -------
+        Response
+            Response from the schedule request.
+        '''
+        return self.__schedule_response
+
+    @property
+    def academic_year(self) -> Optional[str]:
+        '''Current academic year.
+
+        Returns
+        -------
+        Optional[str]
+            Current academic year.
+        '''
+        return self.__academic_year
+
+    @property
+    def semester(self) -> Optional[str]:
+        '''Current semester.
+
+        Returns
+        -------
+        Optional[str]
+            Current semester.
+        '''
+        return self.__semester
+
+    @property
+    def user_data(self) -> dict:
+        '''Return the user data extracted form login response.
+
+        Returns
+        -------
+        dict
+            User data.
+        '''
+        return self.login_response.get('user', {})
+
+    @property
+    def student_data(self) -> dict:
+        '''Return the student data extracted form login response.
+
+        Returns
+        -------
+        dict
+            Student data.
+        '''
+        return self.user_data.get('student', {})
+
+    @property
+    def std_code(self) -> str:
+        '''Return the student code extracted from the login response.
+
+        Returns
+        -------
+        str
+            Student code. (##########)
+        '''
+        return self.student_data.get('stdCode')
+
+    def valid_response(self,
+                       response: Response,
+                       to_json: Optional[dict] = True) -> Union[dict, Response]:
+        '''If the response is not 200, check if the error is due to an expired token. 
+        If so, reset the token and raise an error. Otherwise, raise the error
+
+        Parameters
+        ----------
+        response : Response
+            The response object returned from the request.
+        to_json : Optional[dict]
+            If True, the response will be converted to a JSON object.
+
+        Returns
+        -------
+        Union[dict, Response]
+
+        Raises
+        ------
+        exception.TokenExpired
+            If the response is due to an expired token.
+        exception.HTTPError
+            If the response is not 200.
         '''
 
         data = response.json()
@@ -48,17 +146,11 @@ class Client(ClientType):
         return data if to_json else response
 
     def initialize(self) -> None:
-        '''Initialize the client by logging in and fetch schedule data.
+        '''Initialize the client by logging in and fetch user data.
+        :meth:`login` will be called to fetch login data.
+        After that, :meth:`fetch_schedule` will be called to fetch schedule data.
         
-        Affected attribute
-        -------------------
-        `__login_response`,
-        `__logged_in`,
-        `__access_token`,
-        `__schedule_response`,
-        `__academic_year`,
-        `__semester`
-
+        Affected attributes: :attr:`login_response`, :attr:`.access_token`, :attr:`.schedule_response`, :attr:`.academic_year` and :attr:`.semester`
         '''
 
         self.login()
@@ -69,20 +161,11 @@ class Client(ClientType):
             self.__schedule_response)
 
     def reset(self) -> None:
-        '''Reset the client attribute to NoneType or Negative Value.
-
-        Affected attribute
-        -------------------
-        `__login_response`,
-        `__logged_in`,
-        `__access_token`,
-        `__schedule_response`,
-        `__academic_year`,
-        `__semester`
-            
+        '''Reset the client attributes.
+        
+        Affected attributes: :attr:`login_response`, :attr:`.access_token`, :attr:`.schedule_response`, :attr:`.academic_year` and :attr:`.semester`
         '''
         self.__login_response = {}
-        self.__logged_in = False
         self.__access_token = ''
         self.__schedule_response = {}
         self.__academic_year = None
@@ -93,24 +176,20 @@ class Client(ClientType):
 
         Returns
         -------
-            utils.gen_request_headers(self.__access_token)
-            
+        dict
+            The headers for the requests containing :attr:`pymyku.constant.APP_KEY` and :attr:`access_token`.
         '''
         return utils.gen_request_headers(self.__access_token)
 
     def login(self) -> Response:
-        '''Login to MyKu and fetch the access token.
+        '''Login to MyKu and fetch user data.
 
+        API: https://myapi.ku.th/auth/login
+        
         Returns
         -------
-            Response
-            
-        API
-        ---
-        https://myapi.ku.th/auth/login
-
-        #
-            
+        Response
+            Response object from the login request.
         '''
 
         login_response = requests.request_login(self.__username, self.__password)
@@ -118,29 +197,27 @@ class Client(ClientType):
         login_response.raise_for_status()
 
         self.__login_response = login_response.json()
-        self.__logged_in = True
         self.__access_token = self.__login_response['accesstoken']
 
         return login_response
 
-    def fetch_schedule(self, as_response: bool = False) -> Union[dict, Response]:
+    def fetch_schedule(self,
+                       as_response: Optional[bool] = False) -> Union[dict, Response]:
         '''Send GET request to MyKU common/getschedule API.
 
+        API: https://myapi.ku.th/common/getschedule
+        
         Parameters
         ----------
-        as_response : (bool, optional)
+        as_response : Optional[bool]
             Return as Response object if True, otherwise dict, by default False
 
         Returns
         -------
-            dict | Response 
-            
-        API
-        ---
-        https://myapi.ku.th/common/getschedule
-
-        #
+        Union[dict, Response]
+            Response from the request.
         '''
+
         response = requests.get_schedule(login_response=self.__login_response)
 
         if as_response:
@@ -149,23 +226,20 @@ class Client(ClientType):
         return response.json()
 
     def fetch_group_course(self,
-                           as_response: bool = False) -> Union[List[dict], Response]:
+                           as_response: Optional[bool] = False) -> Union[dict, Response]:
         '''Send GET request to MyKU std-profile/getGroupCourse API.
 
+        API: https://myapi.ku.th/std-profile/getGroupCourse
+        
         Parameters
         ----------
-        as_response : (bool, optional)
+        as_response : Optional[bool]
             Return as Response object if True, otherwise dict, by default False
 
         Returns
         -------
-            List[dict] | Response 
-        
-        API
-        ---
-        https://myapi.ku.th/std-profile/getGroupCourse
-
-        #
+        Union[dict, Response]
+            Response from the request.
         '''
 
         response = requests.get_group_course(login_response=self.__login_response,
@@ -176,23 +250,20 @@ class Client(ClientType):
 
         return response.json()
 
-    def fetch_grades(self, as_response: bool = False) -> List[dict]:
+    def fetch_grades(self, as_response: Optional[bool] = False) -> Union[dict, Response]:
         '''Send GET request to MyKU std-profile/checkGrades API.
 
+        API: https://myapi.ku.th/std-profile/checkGrades
+        
         Parameters
         ----------
-        as_response : (bool, optional)
+        as_response : Optional[bool]
             Return as Response object if True, otherwise dict, by default False
 
         Returns
         -------
-            dict | Response 
-
-        API
-        ---
-        https://myapi.ku.th/std-profile/checkGrades
-
-        #
+        Union[dict, Response]
+            Response from the request.
         '''
 
         response = requests.get_check_grades(login_response=self.__login_response)
@@ -202,23 +273,21 @@ class Client(ClientType):
 
         return response.json()
 
-    def fetch_gpax(self, as_response: bool = False) -> dict:
+    def fetch_gpax(self, as_response: Optional[bool] = False) -> Union[dict, Response]:
         '''Send GET request to MyKU stddashboard/gpax API.
         
+        API: https://myapi.ku.th/stddashboard/gpax
+        
         Parameters
         ----------
-        as_response : (bool, optional)
+        as_response : Optional[bool]
             Return as Response object if True, otherwise dict, by default False
 
         Returns
         -------
-            dict | Response 
+        Union[dict, Response]
+            Response from the request.
 
-        API
-        ---
-        https://myapi.ku.th/stddashboard/gpax
-
-        #
         '''
 
         response = requests.get_gpax(login_response=self.__login_response)
@@ -231,25 +300,22 @@ class Client(ClientType):
     def fetch_annouce(self,
                       academic_year=None,
                       semester=None,
-                      as_response: bool = False) -> Union[List[dict], Response]:
+                      as_response: Optional[bool] = False) -> Union[List[dict], Response]:
         '''Send GET request to MyKU advisor/getAnnounceStd API.
                 
+        API: https://myapi.ku.th/advisor/getAnnounceStd
+        
         Parameters
         ----------
-        as_response : (bool, optional)
+        as_response : Optional[bool]
             Return as Response object if True, otherwise dict, by default False
 
         Returns
         -------
             Union[List[dict], Response]
 
-
-        API
-        ---
-        https://myapi.ku.th/advisor/getAnnounceStd
-
-        #
         '''
+        
         if academic_year is None:
             academic_year = self.__academic_year
 
@@ -267,25 +333,26 @@ class Client(ClientType):
         return response.json()
 
     def search_enroll(self,
-                      academic_year: Union[str, int] = None,
-                      semester: Union[str, int] = None,
-                      as_response: bool = False) -> Union[List[dict], Response]:
+                      academic_year: Optional[Union[str, int]] = None,
+                      semester: Optional[Union[str, int]] = None,
+                      as_response: Optional[bool] = False) -> Union[dict, Response]:
         '''Send GET request to MyKU enroll/searchEnrollResult API.
 
+        API: https://myapi.ku.th/enroll/searchEnrollResult
+        
         Parameters
         ----------
-        as_response : (bool, optional)
+        academic_year : Optional[Union[str, int]]
+            Academic year, by default None
+        semester : Optional[Union[str, int]]
+            Semester, by default None
+        as_response : Optional[bool]
             Return as Response object if True, otherwise dict, by default False
             
         Returns
         -------
-            Union[List[dict], Response]
-
-        API
-        ---
-        https://myapi.ku.th/enroll/searchEnrollResult
-        
-        #
+        Union[dict, Response]
+            Response from the request.
         '''
 
         if isinstance(semester, int):
@@ -304,23 +371,22 @@ class Client(ClientType):
 
         return response.json()
 
-    def fetch_student_personal(self, as_response: bool = False) -> Union[dict, Response]:
+    def fetch_student_personal(self,
+                               as_response: Optional[bool] = False
+                              ) -> Union[dict, Response]:
         '''Send GET request to MyKU std-profile/getStdPersonal API.
 
+        API: https://myapi.ku.th/std-profile/getStdPersonal
+        
         Parameters
         ----------
-        as_response : (bool, optional)
+        as_response : Optional[bool]
             Return as Response object if True, otherwise dict, by default False
 
         Returns
         -------
-            dict | Response
+            Union[dict, Response]
     
-        API
-        ---
-        https://myapi.ku.th/std-profile/getStdPersonal
-        
-        #
         '''
 
         response = requests.get_student_personal(login_response=self.__login_response)
@@ -330,26 +396,24 @@ class Client(ClientType):
 
         return response.json()
 
-    def fetch_student_education(self, as_response: bool = False) -> Union[dict, Response]:
+    def fetch_student_education(self,
+                                as_response: Optional[bool] = False
+                               ) -> Union[dict, Response]:
         '''Send GET request to MyKU std-profile/getStdEducation API.
 
-        Assigning only `login_response` or `client` is acceptable.
-
+        API: https://myapi.ku.th/std-profile/getStdEducation
+        
         Parameters
         ----------
-        as_response : (bool, optional)
+        as_response : Optional[bool]
             Return as Response object if True, otherwise dict, by default False
 
         Returns
         -------
-            dict | Response
-        
-        API
-        ---
-        https://myapi.ku.th/std-profile/getStdEducation
-        
-        #
+        Union[dict, Response]
+            Response from the request.
         '''
+
         response = requests.get_student_education(login_response=self.__login_response)
 
         if as_response:
@@ -357,25 +421,22 @@ class Client(ClientType):
 
         return response.json()
 
-    def fetch_student_address(self, as_response: bool = False) -> Union[dict, Response]:
+    def fetch_student_address(self,
+                              as_response: Optional[bool] = False
+                             ) -> Union[dict, Response]:
         '''Send GET request to MyKU std-profile/getStdAddress API.
 
-        Assigning only `login_response` or `client` is acceptable.
+        API: https://myapi.ku.th/std-profile/getStdAddress
 
         Parameters
         ----------
-        as_response : (bool, optional)
+        as_response : Optional[bool]
             Return as Response object if True, otherwise dict, by default False
 
         Returns
         -------
-            dict | Response
-        
-        API
-        ---
-        https://myapi.ku.th/std-profile/getStdAddress
-        
-        #
+        Union[dict, Response]
+            Response from the request.
         '''
 
         response = requests.get_student_address(login_response=self.__login_response)
@@ -387,21 +448,26 @@ class Client(ClientType):
 
     def search_subject_id(self, subject_id: str) -> List[Dict[str, str]]:
         '''Query subject with subject id by sending GET request to MyKU enroll/searchSubjectOpenEnr API.
+        
+        API: https://myapi.ku.th/enroll/searchSubjectOpenEnr
 
         Parameters
         ----------
-        subject_id : (str)
-            Subject id to query. (At least 3 characters)
-        
+        subject_id : str
+            Subject id to query. (At least 3 characters), e.g. '013' or '01355119'
+
         Returns
         -------
-            List[Dict[str, str]]
+        List[Dict[str, str]]
+            List of subject dicts.
+
+        Raises
+        ------
+        exception.InvalidSubjectID
+            The subject_id is less than 3 characters long.
         
-        API
-        ---
-        https://myapi.ku.th/enroll/searchSubjectOpenEnr
-        
-        #
+        exception.HTTPError
+            The request is not successful.
         '''
 
         if len(subject_id) < 3:
@@ -411,50 +477,63 @@ class Client(ClientType):
         response = requests.search_subject(query=subject_id,
                                            login_response=self.__login_response)
 
-        response = self.__valid_response(response)
+        response = self.valid_response(response)
 
         return response.get('subjects', [])
 
-    def search_subject_open(self, subject_id: str) -> List[Dict[str, Union[str, int]]]:
+    def search_subject_open(self, subject_id: str, section: Optional[str] = '') -> List[Dict[str, Union[str, int]]]:
         '''Query subject enrollment info (All section) of current semester by sending GET request to MyKU enroll/openSubjectForEnroll API.
 
+        API: https://myapi.ku.th/enroll/openSubjectForEnroll
+        
         Parameters
         ----------
-        subject_id : (str)
-            Subject id to query.
+        subject_id : str
+            Subject id to query, e.g. '01355119' or '01355119-64'
+        section : Optional[str]
+            Section of the subject, e.g. '1'
 
         Returns
         -------
-            List[Dict[str, Union[str, int]]]
-        
-        API
-        ---
-        https://myapi.ku.th/enroll/openSubjectForEnroll
-        
-        #
+        List[Dict[str, Union[str, int]]]
+            List of subject's opening section.
+
+        Raises
+        ------
+        exception.InvalidSubjectID
+            The subject_id is less than 3 characters long.
+        exception.HTTPError
+            The request is not successful.
         '''
+
         response = requests.search_subject_open(
             query=subject_id,
+            section=section,
             login_response=self.__login_response,
             schedule_response=self.__schedule_response)
 
-        response = self.__valid_response(response)
+        response = self.valid_response(response)
 
         return response.get('results', [])
 
     def get(self, attr: Enum) -> Any:
         '''Get any value from MyKU client. (login response and schedule response)
-        Use enums from `pymyku.attribute` as key to get value.
+        Use enums from pymyku.attribute as key to get value.
 
         Parameters
         ----------
-        attr : (Enum)
-        
-            Enum from `pymyku.attribute`.
+        attr : Enum
+            Enum from attribute.
 
         Returns
         -------
-            Any
+        Any
+            Value of the attribute.
+            
+        Raises
+        -------
+        TypeError
+            The attr is not an Enum.    
         '''
 
         if isinstance(attr, attribute.FetchedResponses):
@@ -468,75 +547,25 @@ class Client(ClientType):
 
         return utils.extract(self.__login_response, attr)
 
-    def get_login_response(self) -> dict:
-        '''Get login response from MyKU client.
-
-        Returns
-        -------
-            dict
-        '''
-        return self.__login_response
-
-    def get_user_data(self) -> dict:
-        '''Get user data from MyKU client's login response.
-        
-        Returns
-        -------
-            dict
-        '''
-        login_response = self.get_login_response()
-        return login_response.get('user', {})
-
-    def get_student_data(self) -> dict:
-        '''Get student data from MyKU client's login response.
-
-        Returns
-        -------
-            dict
-        '''
-        user_data = self.get_user_data()
-        return user_data.get('student', {})
-
-    def get_schedule_response(self) -> dict:
-        '''Get schedule response from MyKU client.
-        Requires `fetch_schedule` to be called first.
-
-        Returns
-        -------
-            dict
-        '''
-
-        return self.__schedule_response
-
-    def get_access_token(self) -> str:
-        '''Get access token from MyKU client.
-
-        Returns
-        -------
-            str
-        '''
-
-        return self.__access_token
-
-    def get_std_code(self) -> str:
-        '''Get student code from MyKU client.
-
-        Returns
-        -------
-            str
-        '''
-        return self.get(attribute.Student.STD_CODE)
-
     def get_group_course(self) -> List[dict]:
         '''Send GET request to MyKU std-profile/getGroupCourse API and return the result.
 
         Returns
         -------
-            List[dict]
+        List[dict]
+            List of group course dicts (timetable). 
+            
+        Raises
+        -------
+        exception.TokenExpired
+            The token is expired.
+        exception.HTTPError
+            The request is not successful.
         '''
+
         response = self.fetch_group_course(as_response=True)
 
-        response = self.__valid_response(response)
+        response = self.valid_response(response)
 
         return response.get('results', [])
 
@@ -545,35 +574,52 @@ class Client(ClientType):
 
         Returns
         -------
-            float
+        float
+            GPAX.
+            
+        Raises
+        -------
+        exception.TokenExpired
+            The token is expired.
+        exception.HTTPError
+            The request is not successful.
         '''
 
         response = self.fetch_gpax(as_response=True)
 
-        response = self.__valid_response(response)
+        response = self.valid_response(response)
 
-        return response.get('results', {})[0].get('gpax', {})
+        return response.get('results', {})[0].get('gpax')
 
     def get_total_credit(self) -> int:
         '''Send GET request to MyKU std-profile/getTotalCredit API and return the total credit.
 
         Returns
         -------
-            int
+        int
+            Total credit.
+            
+        Raises
+        ------
+        exception.TokenExpired
+            The token is expired.
+        exception.HTTPError
+            The request is not successful.
         '''
 
         response = self.fetch_gpax(as_response=True)
 
-        response = self.__valid_response(response)
+        response = self.valid_response(response)
 
         return response.get('results', {})[0].get('total_credit', None)
 
-    def get_grades(self, key="subject_code") -> Dict[str, Dict[str, str]]:
+    def get_grades(self,
+                   key: Optional[str] = "subject_code") -> Dict[str, Dict[str, str]]:
         '''Fetch grades for each subjects in all semesters.
-
+        
         Parameters
         ----------
-        key  : ((str, optional))
+        key : str
             Subject key from the response. 
             Can be `subject_code` or `subject_name_en`.
             Otherwise, `subject_code` will be used.
@@ -581,15 +627,15 @@ class Client(ClientType):
 
         Returns
         -------
-            Dict[str, Dict[str, str]]:
+        Dict[str, Dict[str, str]]:
+            Dict of grades for each subjects in all semesters.
             
-            {
-                academic_year: {
-                    semester: {
-                        subject_code: grade
-                        }
-                }
-            }
+        Raises
+        -------
+        exception.TokenExpired
+            The token is expired.
+        exception.HTTPError
+            The request is not successful.
         '''
         grades = {}
 
@@ -597,7 +643,7 @@ class Client(ClientType):
 
         response = self.fetch_grades(as_response=True)
 
-        response = self.__valid_response(response)
+        response = self.valid_response(response)
 
         for semester in response['results']:
 
